@@ -26,11 +26,12 @@ from debputy.dh.dh_assistant import (
     read_dh_addon_sequences,
     extract_dh_compat_level,
 )
+from debputy.integration_detection import determine_debputy_integration_mode
 from debputy.packager_provided_files import (
     PackagerProvidedFile,
     detect_all_packager_provided_files,
 )
-from debputy.packages import BinaryPackage
+from debputy.packages import BinaryPackage, SourcePackage
 from debputy.plugin.api import (
     VirtualPath,
     packager_provided_file_reference_documentation,
@@ -45,7 +46,14 @@ from debputy.plugin.api.impl_types import (
     PackagerProvidedFileClassSpec,
     expand_known_packaging_config_features,
 )
-from debputy.util import assume_not_none, escape_shell
+from debputy.plugin.api.spec import DebputyIntegrationMode
+from debputy.util import (
+    assume_not_none,
+    escape_shell,
+    _trace_log,
+    _is_trace_log_enabled,
+    render_command,
+)
 
 PackagingFileInfo = TypedDict(
     "PackagingFileInfo",
@@ -74,6 +82,7 @@ PackagingFileInfo = TypedDict(
 
 def scan_debian_dir(
     feature_set: PluginProvidedFeatureSet,
+    source_package: SourcePackage,
     binary_packages: Mapping[str, BinaryPackage],
     debian_dir: VirtualPath,
     *,
@@ -82,7 +91,6 @@ def scan_debian_dir(
 ) -> Tuple[List[PackagingFileInfo], List[str], int, Optional[object]]:
     known_packaging_files = feature_set.known_packaging_files
     debputy_plugin_metadata = plugin_metadata_for_debputys_own_plugin()
-
     reference_data_set_names = [
         "config-features",
         "file-categories",
@@ -101,12 +109,11 @@ def scan_debian_dir(
         else:
             dh_sequences = set()
             uses_dh_sequencer = False
-    is_debputy_package = (
-        "debputy" in dh_sequences
-        or "zz-debputy" in dh_sequences
-        or "zz_debputy" in dh_sequences
-        or "zz-debputy-rrr" in dh_sequences
+    debputy_integration_mode = determine_debputy_integration_mode(
+        source_package.fields,
+        dh_sequences,
     )
+    is_debputy_package = debputy_integration_mode is not None
     dh_compat_level, dh_assistant_exit_code = extract_dh_compat_level()
     dh_issues = []
 
@@ -150,6 +157,7 @@ def scan_debian_dir(
             dh_sequences,
             dh_compat_level,
             uses_dh_sequencer,
+            debputy_integration_mode=debputy_integration_mode,
             ignore_paths=static_packaging_files,
         )
 
@@ -298,9 +306,12 @@ def resolve_debhelper_config_files(
     dh_compat_level: int,
     saw_dh: bool,
     ignore_paths: Container[str] = frozenset(),
+    *,
+    debputy_integration_mode: Optional[DebputyIntegrationMode] = None,
+    cwd: Optional[str] = None,
 ) -> Tuple[List[PackagerProvidedFile], Optional[object], int]:
     dh_ppfs = {}
-    commands, exit_code = _relevant_dh_commands(dh_rules_addons)
+    commands, exit_code = _relevant_dh_commands(dh_rules_addons, cwd=cwd)
 
     cmd = ["dh_assistant", "list-guessed-dh-config-files"]
     if dh_rules_addons:
@@ -310,6 +321,7 @@ def resolve_debhelper_config_files(
         output = subprocess.check_output(
             cmd,
             stderr=subprocess.DEVNULL,
+            cwd=cwd,
         )
     except (subprocess.CalledProcessError, FileNotFoundError) as e:
         config_files = []
@@ -318,12 +330,20 @@ def resolve_debhelper_config_files(
             exit_code = e.returncode
         else:
             exit_code = 127
+        if _is_trace_log_enabled():
+            _trace_log(
+                f"Command {render_command(*cmd, cwd=cwd)} failed with {exit_code} ({cwd=})"
+            )
     else:
         result = json.loads(output)
         config_files: List[Union[Mapping[str, Any], object]] = result.get(
             "config-files", []
         )
         issues = result.get("issues")
+        if _is_trace_log_enabled():
+            _trace_log(
+                f"Command {render_command(*cmd, cwd=cwd)} returned successfully: {output}"
+            )
     dh_commands = resolve_active_and_inactive_dh_commands(dh_rules_addons)
     for config_file in config_files:
         if not isinstance(config_file, dict):
@@ -389,6 +409,14 @@ def resolve_debhelper_config_files(
                     continue
                 if is_active:
                     has_active_command = True
+
+        if debputy_integration_mode == "full":
+            # dh commands are never active in full integration mode.
+            has_active_command = False
+        elif not saw_dh:
+            # If we did not see `dh`, we assume classic `debhelper` where we have no way of knowing
+            # which commands are active.
+            has_active_command = True
         dh_ppfs[stem] = _fake_PPFClassSpec(
             debputy_plugin_metadata,
             stem,
@@ -398,7 +426,7 @@ def resolve_debhelper_config_files(
             post_formatting_rewrite=post_formatting_rewrite,
             packageless_is_fallback_for_all_packages=packageless_is_fallback_for_all_packages,
             bug_950723=bug_950723,
-            has_active_command=has_active_command if saw_dh else True,
+            has_active_command=has_active_command,
         )
     for ppkpf in dh_ppf_docs.values():
         stem = ppkpf.detection_value
@@ -538,7 +566,10 @@ def _merge_ppfs(
         _merge_list(details, "documentation-uris", documentation_uris)
 
 
-def _relevant_dh_commands(dh_rules_addons: Iterable[str]) -> Tuple[List[str], int]:
+def _relevant_dh_commands(
+    dh_rules_addons: Iterable[str],
+    cwd: Optional[str] = None,
+) -> Tuple[List[str], int]:
     cmd = ["dh_assistant", "list-commands", "--output-format=json"]
     if dh_rules_addons:
         addons = ",".join(dh_rules_addons)
@@ -547,16 +578,25 @@ def _relevant_dh_commands(dh_rules_addons: Iterable[str]) -> Tuple[List[str], in
         output = subprocess.check_output(
             cmd,
             stderr=subprocess.DEVNULL,
+            cwd=cwd,
         )
     except (FileNotFoundError, subprocess.CalledProcessError) as e:
         exit_code = 127
         if isinstance(e, subprocess.CalledProcessError):
             exit_code = e.returncode
+        if _is_trace_log_enabled():
+            _trace_log(
+                f"Command {render_command(*cmd, cwd=cwd)} failed with {exit_code} ({cwd=})"
+            )
         return [], exit_code
     else:
         data = json.loads(output)
         commands_json = data.get("commands")
         commands = []
+        if _is_trace_log_enabled():
+            _trace_log(
+                f"Command {render_command(*cmd, cwd=cwd)} returned successfully: {output}"
+            )
         for command in commands_json:
             if isinstance(command, dict):
                 command_name = command.get("command")
