@@ -44,8 +44,8 @@ from debputy.dh.debhelper_emulation import (
     dhe_install_pkg_file_as_ctrl_file_if_present,
 )
 from debputy.elf_util import find_all_elf_files, ELF_MAGIC
-from debputy.exceptions import DebputyDpkgGensymbolsError
-from debputy.filesystem_scan import FSPath, FSROOverlay
+from debputy.exceptions import DebputyDpkgGensymbolsError, PureVirtualPathError
+from debputy.filesystem_scan import FSPath, FSControlRootDir, VirtualPathBase
 from debputy.maintscript_snippet import (
     ALL_CONTROL_SCRIPTS,
     MaintscriptSnippetContainer,
@@ -103,18 +103,19 @@ _T64_REGEX = re.compile("^lib.*t64(?:-nss)?$")
 _T64_PROVIDES = "t64:Provides"
 
 
-def generate_md5sums_file(control_output_dir: str, fs_root: VirtualPath) -> None:
-    conffiles = os.path.join(control_output_dir, "conffiles")
-    md5sums = os.path.join(control_output_dir, "md5sums")
+def generate_md5sums_file(
+    control_output_dir: VirtualPathBase,
+    fs_root: VirtualPath,
+) -> None:
+    conffiles = control_output_dir.get("conffiles")
     exclude = set()
-    if os.path.isfile(conffiles):
-        with open(conffiles, "rt") as fd:
+    if conffiles and conffiles.is_file:
+        with conffiles.open() as fd:
             for line in fd:
                 if not line.startswith("/"):
                     continue
                 exclude.add("." + line.rstrip("\n"))
-    had_content = False
-    files = sorted(
+    files_to_checksum = sorted(
         (
             path
             for path in fs_root.all_paths()
@@ -125,8 +126,10 @@ def generate_md5sums_file(control_output_dir: str, fs_root: VirtualPath) -> None
         # the two approaches.
         key=lambda p: p.path,
     )
-    with open(md5sums, "wt") as md5fd:
-        for member in files:
+    if not files_to_checksum:
+        return
+    with control_output_dir.open_child("md5sums", "w") as md5fd:
+        for member in files_to_checksum:
             path = member.path
             assert path.startswith("./")
             path = path[2:]
@@ -134,24 +137,21 @@ def generate_md5sums_file(control_output_dir: str, fs_root: VirtualPath) -> None
                 file_hash = hashlib.md5()
                 while chunk := f.read(8192):
                     file_hash.update(chunk)
-            had_content = True
             md5fd.write(f"{file_hash.hexdigest()}  {path}\n")
-    if not had_content:
-        os.unlink(md5sums)
 
 
 def install_or_generate_conffiles(
     binary_package: BinaryPackage,
-    root_dir: str,
+    ctrl_root: str,
     fs_root: VirtualPath,
     debian_dir: VirtualPath,
 ) -> None:
-    conffiles_dest = os.path.join(root_dir, "conffiles")
+    conffiles_dest = os.path.join(ctrl_root, "conffiles")
     dhe_install_pkg_file_as_ctrl_file_if_present(
         debian_dir,
         binary_package,
         "conffiles",
-        root_dir,
+        ctrl_root,
         0o0644,
     )
     etc_dir = fs_root.lookup("etc")
@@ -359,9 +359,10 @@ def _prune_dch_file(
 
     parent_dir = assume_not_none(path.parent_dir)
 
-    with path.replace_fs_path_content() as fs_path, open(
-        fs_path, "wt", encoding="utf-8"
-    ) as fd:
+    with (
+        path.replace_fs_path_content() as fs_path,
+        open(fs_path, "wt", encoding="utf-8") as fd,
+    ):
         for entry in kept_entries:
             fd.write(str(entry))
 
@@ -383,13 +384,16 @@ def _prune_dch_file(
                 " must not include architecture all packages"
             )
 
-        with parent_dir.add_file(
-            f"{path.name}.{package.resolved_architecture}"
-        ) as binnmu_changelog, open(
-            binnmu_changelog.fs_path,
-            "wt",
-            encoding="utf-8",
-        ) as binnmu_fd:
+        with (
+            parent_dir.add_file(
+                f"{path.name}.{package.resolved_architecture}"
+            ) as binnmu_changelog,
+            open(
+                binnmu_changelog.fs_path,
+                "wt",
+                encoding="utf-8",
+            ) as binnmu_fd,
+        ):
             for entry in binnmu_entries:
                 binnmu_fd.write(str(entry))
 
@@ -863,7 +867,6 @@ def cross_package_control_files(
         binary_package = binary_package_data.binary_package
         if binary_package.is_arch_all or not binary_package.should_be_acted_on:
             continue
-        control_output_dir = assume_not_none(binary_package_data.control_output_dir)
         fs_root = binary_package_data.fs_root
         package_state = manifest.package_state_for(binary_package.name)
         related_udeb_package = (
@@ -878,7 +881,7 @@ def cross_package_control_files(
         try:
             soname_info_list = compute_shlibs(
                 binary_package,
-                control_output_dir,
+                binary_package_data.control_output_dir.fs_path,
                 fs_root,
                 manifest,
                 udeb_package_name,
@@ -1094,7 +1097,8 @@ def setup_control_files(
     allow_ctrl_file_management: bool = True,
 ) -> None:
     binary_package = package_metadata_context.binary_package
-    control_output_dir = assume_not_none(binary_package_data.control_output_dir)
+    control_output_dir = binary_package_data.control_output_dir
+    control_output_fs_path = control_output_dir.fs_path
     fs_root = binary_package_data.fs_root
     package_state = manifest.package_state_for(binary_package.name)
 
@@ -1121,7 +1125,7 @@ def setup_control_files(
             package_state.reserved_packager_provided_files,
             package_state.maintscript_snippets,
             substvars,
-            control_output_dir,
+            control_output_fs_path,
         )
 
         handle_service_management(
@@ -1148,7 +1152,7 @@ def setup_control_files(
 
         for script in snippets:
             _generate_snippet(
-                control_output_dir,
+                control_output_fs_path,
                 script,
                 package_state.maintscript_snippets,
             )
@@ -1206,7 +1210,7 @@ def setup_control_files(
         if existing_control_files:
             cmd = ["cp", "-a"]
             cmd.extend(existing_control_files)
-            cmd.append(control_output_dir)
+            cmd.append(control_output_fs_path)
             print_command(*cmd)
             subprocess.check_call(cmd)
 
@@ -1226,7 +1230,7 @@ def setup_control_files(
 
     if generated_triggers:
         assert allow_ctrl_file_management
-        dest_file = os.path.join(control_output_dir, "triggers")
+        dest_file = os.path.join(control_output_fs_path, "triggers")
         with open(dest_file, "at", encoding="utf-8") as fd:
             fd.writelines(
                 textwrap.dedent(
@@ -1242,7 +1246,7 @@ def setup_control_files(
     if allow_ctrl_file_management:
         install_or_generate_conffiles(
             binary_package,
-            control_output_dir,
+            control_output_fs_path,
             fs_root,
             manifest.debian_dir,
         )
@@ -1302,21 +1306,24 @@ def _ensure_base_substvars_defined(substvars: FlushableSubstvars) -> None:
             substvars[substvar] = ""
 
 
-def _compute_installed_size(fs_root: VirtualPath) -> int:
+def compute_installed_size(fs_root: VirtualPath) -> int:
     """Emulate dpkg-gencontrol's code for computing the default Installed-Size"""
     size_in_kb = 0
     hard_links = set()
     for path in fs_root.all_paths():
-        if not path.is_dir and path.has_fs_path:
-            st = path.stat()
-            if st.st_nlink > 1:
-                hl_key = (st.st_dev, st.st_ino)
-                if hl_key in hard_links:
-                    continue
-                hard_links.add(hl_key)
-            path_size = (st.st_size + 1023) // 1024
-        elif path.is_symlink:
-            path_size = (len(path.readlink()) + 1023) // 1024
+        if path.is_symlink or path.is_file:
+            try:
+                st = path.stat()
+                if st.st_nlink > 1:
+                    hl_key = (st.st_dev, st.st_ino)
+                    if hl_key in hard_links:
+                        continue
+                    hard_links.add(hl_key)
+                size = st.st_size
+            except PureVirtualPathError:
+                # We just assume it is not a hard link when the path is purely virtual
+                size = path.size
+            path_size = (size + 1023) // 1024
         else:
             path_size = 1
         size_in_kb += path_size
@@ -1326,7 +1333,7 @@ def _compute_installed_size(fs_root: VirtualPath) -> int:
 def _generate_dbgsym_control_file_if_relevant(
     binary_package: BinaryPackage,
     dbgsym_fs_root: VirtualPath,
-    dbgsym_control_dir: str,
+    dbgsym_control_dir: FSControlRootDir,
     dbgsym_ids: str,
     multi_arch: Optional[str],
     dctrl: str,
@@ -1343,12 +1350,12 @@ def _generate_dbgsym_control_file_if_relevant(
         extra_params.append(f"-DMulti-Arch={multi_arch}")
     extra_params.append("-UReplaces")
     extra_params.append("-UBreaks")
-    ensure_dir(dbgsym_control_dir)
+    dbgsym_control_fs_path = dbgsym_control_dir.fs_path
+    ensure_dir(dbgsym_control_fs_path)
     # Pass it via cmd-line to make it more visible that we are providing the
     # value.  It also prevents the dbgsym package from picking up this value.
-    ctrl_fs_root = FSROOverlay.create_root_dir("DEBIAN", dbgsym_control_dir)
-    total_size = _compute_installed_size(dbgsym_fs_root) + _compute_installed_size(
-        ctrl_fs_root
+    total_size = compute_installed_size(dbgsym_fs_root) + compute_installed_size(
+        dbgsym_control_dir
     )
     extra_params.append(f"-VInstalled-Size={total_size}")
     extra_params.extend(extra_common_params)
@@ -1366,7 +1373,7 @@ def _generate_dbgsym_control_file_if_relevant(
         "-ldebian/changelog",
         "-T/dev/null",
         f"-c{dctrl}",
-        f"-O{dbgsym_control_dir}/control",
+        f"-O{dbgsym_control_fs_path}/control",
         # Use a placeholder for -P to ensure failure if we forgot to override a path parameter
         "-P/non-existent",
         f"-DPackage={package}-dbgsym",
@@ -1398,7 +1405,7 @@ def _generate_dbgsym_control_file_if_relevant(
             f"Attempting to generate DEBIAN/control file for {package}-dbgsym failed. Please review the output from "
             " dpkg-gencontrol above to understand what went wrong."
         )
-    os.chmod(os.path.join(dbgsym_control_dir, "control"), 0o644)
+    os.chmod(os.path.join(dbgsym_control_fs_path, "control"), 0o644)
 
 
 def _all_parent_directories_of(directories: Iterable[str]) -> Set[str]:
@@ -1454,16 +1461,16 @@ def _any_unacceptable_paths(
     return False
 
 
-def _auto_compute_multi_arch(
+def auto_compute_multi_arch(
     binary_package: BinaryPackage,
-    control_output_dir: str,
+    control_output_dir: VirtualPath,
     fs_root: FSPath,
 ) -> Optional[str]:
     resolved_arch = binary_package.resolved_architecture
     if any(
         script
         for script in ALL_CONTROL_SCRIPTS
-        if os.path.isfile(os.path.join(control_output_dir, script))
+        if (p := control_output_dir.get(script)) is not None and p.is_file
     ):
         return None
 
@@ -1522,7 +1529,7 @@ def _has_t64_enabled() -> bool:
 
 def _t64_migration_substvar(
     binary_package: BinaryPackage,
-    control_output_dir: str,
+    control_output_dir: VirtualPath,
     substvars: FlushableSubstvars,
 ) -> None:
     name = binary_package.name
@@ -1531,8 +1538,9 @@ def _t64_migration_substvar(
         return
 
     if not any(
-        os.path.isfile(os.path.join(control_output_dir, n))
+        p.is_file
         for n in ["symbols", "shlibs"]
+        if (p := control_output_dir.get(n)) is not None
     ):
         return
 
@@ -1652,7 +1660,7 @@ def _handle_relationship_substvars(
 def _generate_control_files(
     binary_package_data: "BinaryPackageData",
     package_state: "PackageTransformationDefinition",
-    control_output_dir: str,
+    control_output_dir: FSControlRootDir,
     fs_root: FSPath,
     substvars: FlushableSubstvars,
     dbgsym_root_fs: Optional[VirtualPath],
@@ -1667,15 +1675,14 @@ def _generate_control_files(
     if "Installed-Size" not in substvars:
         # Pass it via cmd-line to make it more visible that we are providing the
         # value.  It also prevents the dbgsym package from picking up this value.
-        ctrl_fs_root = FSROOverlay.create_root_dir("DEBIAN", control_output_dir)
-        total_size = _compute_installed_size(fs_root) + _compute_installed_size(
-            ctrl_fs_root
+        total_size = compute_installed_size(fs_root) + compute_installed_size(
+            control_output_dir
         )
         extra_params_specific.append(f"-VInstalled-Size={total_size}")
 
     ma_value = binary_package.fields.get("Multi-Arch")
     if not binary_package.is_udeb and ma_value is None:
-        ma_value = _auto_compute_multi_arch(binary_package, control_output_dir, fs_root)
+        ma_value = auto_compute_multi_arch(binary_package, control_output_dir, fs_root)
         if ma_value is not None:
             _info(
                 f'The package "{binary_package.name}" looks like it should be "Multi-Arch: {ma_value}" based'
@@ -1727,7 +1734,7 @@ def _generate_control_files(
         elif dbgsym_ids:
             extra_common_params.append(f"-DBuild-Ids={dbgsym_ids}")
 
-        ctrl_file = os.path.join(control_output_dir, "control")
+        ctrl_file = os.path.join(control_output_dir.fs_path, "control")
         dpkg_cmd = [
             "dpkg-gencontrol",
             f"-p{package_name}",
