@@ -26,7 +26,7 @@ class DhSequencerData:
 
 
 def _parse_dh_cmd_list(
-    cmd_list: Optional[List[Union[Mapping[str, Any], object]]]
+    cmd_list: Optional[List[Union[Mapping[str, Any], object]]],
 ) -> Iterable[str]:
     if not isinstance(cmd_list, list):
         return
