@@ -37,7 +37,7 @@ from .exceptions import (
     DebputyRuntimeError,
     DebputyRuntimeErrorWithPreamble,
 )
-from .filesystem_scan import FSPath, FSRootDir, FSROOverlay
+from .filesystem_scan import FSPath, FSRootDir, FSROOverlay, FSControlRootDir
 from .installations import (
     InstallRule,
     SourcePathMatcher,
@@ -135,8 +135,10 @@ class DbgsymInfo:
         return root_dir
 
     @property
-    def dbgsym_ctrl_dir(self) -> str:
-        return os.path.join(self.dbgsym_root_dir, "DEBIAN")
+    def dbgsym_ctrl_dir(self) -> FSControlRootDir:
+        return FSControlRootDir.create_root_dir(
+            os.path.join(self.dbgsym_root_dir, "DEBIAN")
+        )
 
 
 @dataclass(slots=True, frozen=True)
@@ -151,10 +153,12 @@ class BinaryPackageData:
     dbgsym_info: DbgsymInfo
 
     @property
-    def control_output_dir(self) -> str:
-        return generated_content_dir(
-            package=self.binary_package,
-            subdir_key="DEBIAN",
+    def control_output_dir(self) -> FSControlRootDir:
+        return FSControlRootDir.create_root_dir(
+            generated_content_dir(
+                package=self.binary_package,
+                subdir_key="DEBIAN",
+            )
         )
 
 
