@@ -585,7 +585,7 @@ async def _auto_fix_run(
         lint_state.content = text
         lint_state.lines = lines
         lint_state.clear_cache()
-        current_issues = lint_state.gather_diagnostics()
+        current_issues = await lint_state.gather_diagnostics()
 
     if fixed_count:
         output_filename = f"{filename}.tmp"
@@ -597,7 +597,7 @@ async def _auto_fix_run(
         lines = text.splitlines(keepends=True)
         lint_state.content = text
         lint_state.lines = lines
-        remaining_issues = lint_state.gather_diagnostics() or []
+        remaining_issues = await lint_state.gather_diagnostics() or []
     else:
         remaining_issues = current_issues or []
 
