@@ -20,6 +20,8 @@ from typing import (
     Set,
     Dict,
     Iterable,
+    Awaitable,
+    AsyncIterable,
 )
 
 import debputy.l10n as l10n
@@ -29,7 +31,7 @@ from debputy.dh.dh_assistant import (
     DhSequencerData,
     parse_drules_for_addons,
 )
-from debputy.exceptions import PureVirtualPathError
+from debputy.exceptions import PureVirtualPathError, DebputyRuntimeError
 from debputy.filesystem_scan import VirtualPathBase
 from debputy.integration_detection import determine_debputy_integration_mode
 from debputy.l10n import Translations
@@ -44,7 +46,7 @@ from debputy.lsp.vendoring._deb822_repro import Deb822FileElement, parse_deb822_
 from debputy.packages import SourcePackage, BinaryPackage
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.spec import DebputyIntegrationMode
-from debputy.util import _warn
+from debputy.util import _warn, T
 
 if TYPE_CHECKING:
     import lsprotocol.types as types
@@ -61,10 +63,14 @@ else:
     import debputy.lsprotocol.types as types
 
 
-LinterImpl = Callable[["LintState"], None]
+AsyncLinterImpl = Callable[["LintState"], Awaitable[None]]
 FormatterImpl = Callable[["LintState"], Optional[Sequence[types.TextEdit]]]
 
 
+class AbortTaskError(DebputyRuntimeError):
+    pass
+
+
 # If you add a new one to this set, remember to mention it in the docs of `LintState.emit_diagnostic`
 DIAG_SOURCE_WITHOUT_SECTIONS: FrozenSet[str] = frozenset(
     {
@@ -129,6 +135,25 @@ class DebputyMetadata:
         return cls(integration_mode)
 
 
+@dataclasses.dataclass(slots=True, frozen=True)
+class RelatedDiagnosticInformation:
+    text_range: "TERange"
+    message: str
+    doc_uri: str
+
+    def to_lsp(self, lint_state: "LintState") -> types.DiagnosticRelatedInformation:
+        return types.DiagnosticRelatedInformation(
+            types.Location(
+                self.doc_uri,
+                lint_state.position_codec.range_to_client_units(
+                    lint_state.lines,
+                    te_range_to_lsp(self.text_range),
+                ),
+            ),
+            self.message,
+        )
+
+
 class LintState:
 
     @property
@@ -257,11 +282,46 @@ class LintState:
             ignored_words.update(binary_packages)
         return checker.context_ignored_words(ignored_words)
 
+    async def slow_iter(
+        self,
+        iterable: Iterable[T],
+        *,
+        yield_every: int = 100,
+    ) -> AsyncIterable[T]:
+        for value in iterable:
+            yield value
+
     def translation(self, domain: str) -> Translations:
         return l10n.translation(
             domain,
         )
 
+    def related_diagnostic_information(
+        self,
+        text_range: "TERange",
+        message: str,
+        *,
+        doc_uri: Optional[str] = None,
+    ) -> RelatedDiagnosticInformation:
+        """Provide a related context for the diagnostic
+
+        The related diagnostic information is typically highlighted with the diagnostic. As an example,
+        `debputy lint`'s terminal output will display the message and display the selected range after the
+        the diagnostic itself.
+
+        :param text_range: The text range to highlight.
+        :param message: The message to associate with the provided text range.
+        :param doc_uri: The URI of the document that the context is from. When omitted, the text range is
+          assumed to be from the "current" file (the `doc_uri` attribute), which is also the default file
+          for ranges passed to `emit_diagnostic`.
+        :return:
+        """
+        return RelatedDiagnosticInformation(
+            text_range,
+            message,
+            doc_uri=doc_uri if doc_uri is not None else self.doc_uri,
+        )
+
     def emit_diagnostic(
         self,
         text_range: "TERange",
@@ -271,7 +331,7 @@ class LintState:
         *,
         quickfixes: Optional[List[dict]] = None,
         tags: Optional[List[types.DiagnosticTag]] = None,
-        related_information: Optional[List[types.DiagnosticRelatedInformation]] = None,
+        related_information: Optional[List[RelatedDiagnosticInformation]] = None,
         diagnostic_applies_to_another_file: Optional[str] = None,
         enable_non_interactive_auto_fix: bool = True,
     ) -> None:
@@ -284,7 +344,9 @@ class LintState:
 
             Use:
               * "Policy 3.4.1" for Debian Policy Manual section 3.4.1
+                (replace the section number with the relevant number for your case)
               * "DevRef 6.2.2" for the Debian Developer Reference section 6.2.2
+                (replace the section number with the relevant number for your case)
               * "debputy" for diagnostics without a reference or where `debputy` is the authority.
                 (This is also used for cases where `debputy` filters the result. Like with spellchecking
                  via hunspell, where `debputy` provides its own ignore list on top)
@@ -293,7 +355,11 @@ class LintState:
         :param quickfixes: If provided, this is a list of possible fixes for this problem.
           Use the quickfixes provided in `debputy.lsp.quickfixes` such as `propose_correct_text_quick_fix`.
         :param tags: TODO: Not yet specified (currently uses LSP format).
-        :param related_information: TODO: Not yet specified (currently uses LSP format).
+        :param related_information: Provide additional context to the diagnostic. This can be used to define
+          the source of a conflict. As an example, for duplicate definitions, this can be used to show where
+          the definitions are.
+
+          Every item should be created via the `related_diagnostic_information` method.
         :param enable_non_interactive_auto_fix: Allow non-interactive auto-fixing (such as via
           `debputy lint --auto-fix`) of this issue. Set to `False` if the check is likely to have false
           positives.
@@ -321,6 +387,11 @@ class LintState:
             te_range_to_lsp(text_range),
         )
 
+        related_lsp_format = (
+            [i.to_lsp(self) for i in related_information]
+            if related_information
+            else None
+        )
         diag = types.Diagnostic(
             lsp_range_client_units,
             diagnostic_msg,
@@ -328,7 +399,7 @@ class LintState:
             source=authority_reference,
             data=diag_data if diag_data else None,
             tags=tags,
-            related_information=related_information,
+            related_information=related_lsp_format,
         )
         self._emit_diagnostic(diag)
 
@@ -348,7 +419,7 @@ class LintStateImpl(LintState):
     source_package: Optional[SourcePackage] = None
     binary_packages: Optional[Mapping[str, BinaryPackage]] = None
     effective_preference: Optional["EffectiveFormattingPreference"] = None
-    lint_implementation: Optional["LinterImpl"] = None
+    lint_implementation: Optional["AsyncLinterImpl"] = None
     _parsed_cache: Optional[Deb822FileElement] = None
     _dh_sequencer_cache: Optional[DhSequencerData] = None
     _diagnostics: Optional[List[types.Diagnostic]] = None
@@ -400,7 +471,7 @@ class LintStateImpl(LintState):
             self._dh_sequencer_cache = dh_sequencer_cache
         return dh_sequencer_cache
 
-    def gather_diagnostics(self) -> List[types.Diagnostic]:
+    async def gather_diagnostics(self) -> List[types.Diagnostic]:
         if self._diagnostics is not None:
             raise RuntimeError(
                 "run_diagnostics cannot be run while it is already running"
@@ -413,7 +484,7 @@ class LintStateImpl(LintState):
             )
         self._diagnostics = diagnostics = []
 
-        linter(self)
+        await linter(self)
 
         self._diagnostics = None
         return diagnostics
@@ -663,7 +734,7 @@ class TermLintReport(LintReport):
         assert authority is not None
         diag_tags = f" [{authority}]"
         lines = lint_state.lines
-        line_no_width = len(str(len(lines)))
+        line_no_format_width = len(str(len(lines)))
 
         if diagnostic_result.result_state == LintDiagnosticResultState.AUTO_FIXABLE:
             diag_tags += "[Correctable via --auto-fix]"
@@ -691,10 +762,34 @@ class TermLintReport(LintReport):
                 "Bug in the underlying linter: The line numbers of the warning does not fit in the file..."
             )
             return
-        lines_to_print = _lines_to_print(diagnostic.range)
+        self._print_range_context(diagnostic.range, lines, line_no_format_width)
+        related_info_list = diagnostic.related_information or []
+        hint_tag = fo.colored(
+            "Related information",
+            fg="magenta",
+            bg="black",
+            style="bold",
+        )
+        for related_info in related_info_list:
+            if related_info.location.uri != lint_state.doc_uri:
+                continue
+            print(f"  {hint_tag}: {related_info.message}")
+            self._print_range_context(
+                related_info.location.range, lines, line_no_format_width
+            )
+
+    def _print_range_context(
+        self,
+        print_range: types.Range,
+        lines: List[str],
+        line_no_format_width: int,
+    ) -> None:
+        lines_to_print = _lines_to_print(print_range)
+        fo = self.fo
+        start_line = print_range.start.line
         for line_no in range(start_line, start_line + lines_to_print):
-            line = _highlight_range(fo, lines[line_no], line_no, diagnostic.range)
-            print(f"    {line_no+1:{line_no_width}}: {line}")
+            line = _highlight_range(fo, lines[line_no], line_no, print_range)
+            print(f"    {line_no + 1:{line_no_format_width}}: {line}")
 
 
 class LinterPositionCodec:
