@@ -1,6 +1,8 @@
+import asyncio
 import dataclasses
 import os
 import time
+from collections.abc import AsyncIterable
 from typing import (
     Optional,
     List,
@@ -12,6 +14,7 @@ from typing import (
     Literal,
     Set,
     Dict,
+    Iterable,
 )
 
 import debputy.l10n as l10n
@@ -24,7 +27,8 @@ from debputy.filesystem_scan import FSROOverlay, VirtualPathBase
 from debputy.l10n import Translations
 from debputy.linting.lint_util import (
     LintState,
-    LinterImpl,
+    AsyncLinterImpl,
+    AbortTaskError,
 )
 from debputy.lsp.apt_cache import AptCache
 from debputy.lsp.diagnostics import DiagnosticReport
@@ -41,7 +45,7 @@ from debputy.packages import (
     DctrlParser,
 )
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
-from debputy.util import _info, _warn
+from debputy.util import _info, _warn, T
 from debputy.yaml import MANIFEST_YAML, YAMLError
 from debputy.yaml.compat import CommentedMap
 
@@ -214,6 +218,7 @@ class LSProvidedLintState(LintState):
             debian_dir = None
         self._debian_dir = debian_dir
         self._diagnostics: Optional[List[types.Diagnostic]] = None
+        self._last_emitted_diagnostic_count = 0
         dctrl_file = os.path.join(debian_dir_path, "control")
 
         if dctrl_file != doc.path:
@@ -346,9 +351,27 @@ class LSProvidedLintState(LintState):
     def translation(self, domain: str) -> Translations:
         return self._ls.translation(domain)
 
-    def run_diagnostics(
+    async def slow_iter(
         self,
-        linter: LinterImpl,
+        iterable: Iterable[T],
+        *,
+        yield_every: int = 100,
+    ) -> AsyncIterable[T]:
+        counter = 0
+        for value in iterable:
+            counter += 1
+            if counter >= yield_every:
+                await asyncio.sleep(0)
+                self._abort_on_outdated_doc_version()
+                counter = 0
+            yield value
+        if counter:
+            await asyncio.sleep(0)
+            self._abort_on_outdated_doc_version()
+
+    async def run_diagnostics(
+        self,
+        linter: AsyncLinterImpl,
     ) -> List[types.Diagnostic]:
         if self._diagnostics is not None:
             raise RuntimeError(
@@ -356,7 +379,7 @@ class LSProvidedLintState(LintState):
             )
         self._diagnostics = diagnostics = []
 
-        linter(self)
+        await linter(self)
 
         self._diagnostics = None
         return diagnostics
@@ -365,7 +388,26 @@ class LSProvidedLintState(LintState):
         diagnostics = self._diagnostics
         if diagnostics is None:
             raise TypeError("Cannot run emit_diagnostic outside of run_diagnostics")
+
         diagnostics.append(diagnostic)
+        self._last_emitted_diagnostic_count += 1
+        if self._last_emitted_diagnostic_count >= 100:
+            self._abort_on_outdated_doc_version()
+            self._ls.record_diagnostics(
+                self.doc_uri,
+                self._doc.version,
+                diagnostics,
+                is_partial=True,
+            )
+            self._last_emitted_diagnostic_count = 0
+
+    def _abort_on_outdated_doc_version(self) -> None:
+        expected_version = self._doc.version
+        current_doc = self._ls.workspace.get_text_document(self.doc_uri)
+        if current_doc.version != expected_version:
+            raise AbortTaskError(
+                f"Cancel (obsolete) diagnostics for doc version {expected_version}: document version changed"
+            )
 
 
 def _preference(
@@ -426,6 +468,7 @@ class DebputyLanguageServer(LanguageServer):
             _info(
                 f"Client locale: {self.client_locale}. Use --force-locale to override"
             )
+        _info(f"Cwd: {os.getcwd()}")
         self._update_locale()
 
     def _update_locale(self) -> None:
@@ -600,19 +643,37 @@ class DebputyLanguageServer(LanguageServer):
         except KeyError:
             pass
 
+    async def slow_iter(
+        self,
+        iterable: Iterable[T],
+        *,
+        yield_every: int = 100,
+    ) -> AsyncIterable[T]:
+        counter = 0
+        for value in iterable:
+            counter += 1
+            if counter >= yield_every:
+                await asyncio.sleep(0)
+                counter = 0
+            yield value
+        if counter:
+            await asyncio.sleep(0)
+
     def record_diagnostics(
         self,
-        uri: str,
+        doc_uri: str,
         version: int,
         diagnostics: List[types.Diagnostic],
+        is_partial: bool,
     ) -> None:
-        self._diagnostic_reports[uri] = DiagnosticReport(
-            uri,
+        self._diagnostic_reports[doc_uri] = DiagnosticReport(
+            doc_uri,
             version,
-            f"{version}@{uri}",
-            False,
+            f"{version}@{doc_uri}",
+            is_partial,
             diagnostics,
         )
+        self.publish_diagnostics(doc_uri, diagnostics, version)
 
     def diagnostics_in_range(
         self,
