@@ -1,3 +1,4 @@
+import asyncio
 import dataclasses
 import importlib.resources
 import os.path
@@ -55,8 +56,8 @@ from debputy.lsp.lsp_generic_deb822 import (
     deb822_hover,
     deb822_folding_ranges,
     deb822_semantic_tokens_full,
-    deb822_token_iter,
     deb822_format_file,
+    scan_for_syntax_errors_and_token_level_diagnostics,
 )
 from debputy.lsp.lsp_reference_keyword import LSP_DATA_DOMAIN
 from debputy.lsp.quickfixes import (
@@ -70,7 +71,6 @@ from debputy.lsp.ref_models.deb822_reference_parse_models import (
 )
 from debputy.lsp.text_util import markdown_urlify
 from debputy.lsp.vendoring._deb822_repro import (
-    Deb822FileElement,
     Deb822ParagraphElement,
 )
 from debputy.lsp.vendoring._deb822_repro.parsing import (
@@ -99,7 +99,7 @@ from debputy.packager_provided_files import (
     detect_all_packager_provided_files,
 )
 from debputy.plugin.api.impl import plugin_metadata_for_debputys_own_plugin
-from debputy.util import PKGNAME_REGEX, _info
+from debputy.util import PKGNAME_REGEX, _info, _trace_log, _is_trace_log_enabled
 from debputy.yaml import MANIFEST_YAML
 
 if TYPE_CHECKING:
@@ -598,7 +598,7 @@ def _debian_control_folding_ranges(
 
 
 @lsp_text_doc_inlay_hints(_DISPATCH_RULE)
-def _doc_inlay_hint(
+async def _doc_inlay_hint(
     ls: "DebputyLanguageServer",
     params: types.InlayHintParams,
 ) -> Optional[List[InlayHint]]:
@@ -613,7 +613,7 @@ def _doc_inlay_hint(
         return None
     source_stanza = stanzas[0]
     source_stanza_pos = source_stanza.position_in_file()
-    for stanza_no, stanza in enumerate(deb822_file):
+    async for stanza_no, stanza in ls.slow_iter(enumerate(deb822_file), yield_every=20):
         stanza_range = stanza.range_in_parent()
         if stanza_no < 1:
             continue
@@ -622,7 +622,9 @@ def _doc_inlay_hint(
             continue
         stanza_def = _DCTRL_FILE_METADATA.classify_stanza(stanza, stanza_no)
         parts = []
-        for known_field in stanza_def.stanza_fields.values():
+        async for known_field in ls.slow_iter(
+            stanza_def.stanza_fields.values(), yield_every=25
+        ):
             if (
                 not known_field.inheritable_from_other_stanza
                 or not known_field.show_as_inherited
@@ -782,66 +784,11 @@ def _binary_package_checks(
         )
 
 
-def _scan_for_syntax_errors_and_token_level_diagnostics(
-    deb822_file: Deb822FileElement,
-    lint_state: LintState,
-) -> int:
-    first_error = len(lint_state.lines) + 1
-    spell_checker = lint_state.spellchecker()
-    for (
-        token,
-        start_line,
-        start_offset,
-        end_line,
-        end_offset,
-    ) in deb822_token_iter(deb822_file.iter_tokens()):
-        if token.is_error:
-            first_error = min(first_error, start_line)
-            start_pos = TEPosition(
-                start_line,
-                start_offset,
-            )
-            end_pos = TEPosition(
-                end_line,
-                end_offset,
-            )
-            token_range = TERange.between(start_pos, end_pos)
-            lint_state.emit_diagnostic(
-                token_range,
-                "Syntax error",
-                "error",
-                "debputy",
-            )
-        elif token.is_comment:
-            for word, col_pos, end_col_pos in spell_checker.iter_words(token.text):
-                corrections = spell_checker.provide_corrections_for(word)
-                if not corrections:
-                    continue
-                start_pos = TEPosition(
-                    start_line,
-                    col_pos,
-                )
-                end_pos = TEPosition(
-                    start_line,
-                    end_col_pos,
-                )
-                word_range = TERange.between(start_pos, end_pos)
-                lint_state.emit_diagnostic(
-                    word_range,
-                    f'Spelling "{word}"',
-                    "spelling",
-                    "debputy",
-                    quickfixes=[propose_correct_text_quick_fix(c) for c in corrections],
-                    enable_non_interactive_auto_fix=False,
-                )
-    return first_error
-
-
 @lint_diagnostics(_DISPATCH_RULE)
-def _lint_debian_control(lint_state: LintState) -> None:
+async def _lint_debian_control(lint_state: LintState) -> None:
     deb822_file = lint_state.parsed_deb822_file_content
 
-    first_error = _scan_for_syntax_errors_and_token_level_diagnostics(
+    first_error = await scan_for_syntax_errors_and_token_level_diagnostics(
         deb822_file,
         lint_state,
     )
@@ -852,7 +799,7 @@ def _lint_debian_control(lint_state: LintState) -> None:
 
     source_stanza_metadata, binary_stanza_metadata = _DCTRL_FILE_METADATA.stanza_types()
 
-    for stanza_no, stanza in enumerate(stanzas, start=1):
+    async for stanza_no, stanza in lint_state.slow_iter(enumerate(stanzas, start=1)):
         stanza_position = stanza.position_in_file()
         if stanza_position.line_position >= first_error:
             break
@@ -883,7 +830,7 @@ def _lint_debian_control(lint_state: LintState) -> None:
                 lint_state,
             )
 
-        stanza_metadata.stanza_diagnostics(
+        await stanza_metadata.stanza_diagnostics(
             deb822_file,
             stanza,
             stanza_position,
@@ -926,14 +873,10 @@ def _packaging_files(
     ):
         return
 
+    debputy_integration_mode = lint_state.debputy_metadata.debputy_integration_mode
     dh_sequencer_data = lint_state.dh_sequencer_data
     dh_sequences = dh_sequencer_data.sequences
-    is_debputy_package = (
-        "debputy" in dh_sequences
-        or "zz-debputy" in dh_sequences
-        or "zz_debputy" in dh_sequences
-        or "zz-debputy-rrr" in dh_sequences
-    )
+    is_debputy_package = debputy_integration_mode is not None
     feature_set = lint_state.plugin_feature_set
     known_packaging_files = feature_set.known_packaging_files
     static_packaging_files = {
@@ -984,6 +927,8 @@ def _packaging_files(
             dh_compat_level,
             saw_dh=dh_sequencer_data.uses_dh_sequencer,
             ignore_paths=ignored_path,
+            debputy_integration_mode=debputy_integration_mode,
+            cwd=source_root.fs_path,
         )
         for ppf in all_dh_ppfs:
             if ppf.path.path in ignored_path:
@@ -1004,6 +949,10 @@ def _detect_misspelled_packaging_files(
         explicit_package = ppf.uses_explicit_package_name
         name_segment = ppf.name_segment is not None
         stem = ppf.definition.stem
+        if _is_trace_log_enabled():
+            _trace_log(
+                f"PPF check: {binary_package} {stem=} {explicit_package=} {name_segment=} {ppf.expected_path=} {ppf.definition.has_active_command=}"
+            )
         if binary_package is None or stem is None:
             continue
         res = stanza_ranges.get(binary_package)
@@ -1104,11 +1053,11 @@ def _debian_control_format_file(
 
 
 @lsp_semantic_tokens_full(_DISPATCH_RULE)
-def _debian_control_semantic_tokens_full(
+async def _debian_control_semantic_tokens_full(
     ls: "DebputyLanguageServer",
     request: SemanticTokensParams,
 ) -> Optional[SemanticTokens]:
-    return deb822_semantic_tokens_full(
+    return await deb822_semantic_tokens_full(
         ls,
         request,
         _DCTRL_FILE_METADATA,
