@@ -199,7 +199,7 @@ def _forbidden_hook_targets(dh_commands: DhListCommands) -> FrozenSet[str]:
 
 
 @lint_diagnostics(_DISPATCH_RULE)
-def _lint_debian_rules(lint_state: LintState) -> None:
+async def _lint_debian_rules(lint_state: LintState) -> None:
     lines = lint_state.lines
     path = lint_state.path
     source_root = os.path.dirname(os.path.dirname(path))
