@@ -607,6 +607,42 @@ def extract_maint_email(maint: str) -> str:
     return maint[idx + 1 : -1]
 
 
+def _parse_salsa_ci_boolean(value: Union[str, int, bool]) -> bool:
+    if isinstance(value, str):
+        return value in ("yes", "1", "true")
+    elif not isinstance(value, (int, bool)):
+        raise TypeError("Unsupported value")
+    else:
+        return value is True or value == 1
+
+
+def _read_salsa_ci_wrap_and_sort_enabled(salsa_ci: Optional[CommentedMap]) -> bool:
+    sentinel = object()
+    disable_wrap_and_sort_raw = salsa_ci.mlget(
+        ["variables", "SALSA_CI_DISABLE_WRAP_AND_SORT"],
+        list_ok=True,
+        default=sentinel,
+    )
+
+    if disable_wrap_and_sort_raw is sentinel:
+        enable_wrap_and_sort_raw = salsa_ci.mlget(
+            ["variables", "SALSA_CI_ENABLE_WRAP_AND_SORT"],
+            list_ok=True,
+            default=None,
+        )
+        if enable_wrap_and_sort_raw is None or not isinstance(
+            enable_wrap_and_sort_raw, (str, int, bool)
+        ):
+            return False
+
+        return _parse_salsa_ci_boolean(enable_wrap_and_sort_raw)
+    if not isinstance(disable_wrap_and_sort_raw, (str, int, bool)):
+        return False
+
+    disable_wrap_and_sort = _parse_salsa_ci_boolean(disable_wrap_and_sort_raw)
+    return not disable_wrap_and_sort
+
+
 def determine_effective_preference(
     maint_preference_table: MaintainerPreferenceTable,
     source_package: Optional[SourcePackage],
@@ -618,42 +654,27 @@ def determine_effective_preference(
             return None, None, "X-Style contained an unknown/unsupported style"
         return maint_preference_table.named_styles.get(style), "debputy reformat", None
 
-    if salsa_ci:
-        disable_wrap_and_sort = salsa_ci.mlget(
-            ["variables", "SALSA_CI_DISABLE_WRAP_AND_SORT"],
+    if salsa_ci and _read_salsa_ci_wrap_and_sort_enabled(salsa_ci):
+        wrap_and_sort_options = salsa_ci.mlget(
+            ["variables", "SALSA_CI_WRAP_AND_SORT_ARGS"],
             list_ok=True,
-            default=True,
+            default=None,
         )
-
-        if isinstance(disable_wrap_and_sort, str):
-            disable_wrap_and_sort = disable_wrap_and_sort in ("yes", "1", "true")
-        elif not isinstance(disable_wrap_and_sort, (int, bool)):
-            disable_wrap_and_sort = True
+        if wrap_and_sort_options is None:
+            wrap_and_sort_options = ""
+        elif not isinstance(wrap_and_sort_options, str):
+            return (
+                None,
+                None,
+                "The salsa-ci had a non-string option for wrap-and-sort",
+            )
+        detected_style = parse_salsa_ci_wrap_and_sort_args(wrap_and_sort_options)
+        tool_w_args = f"wrap-and-sort {wrap_and_sort_options}".strip()
+        if detected_style is None:
+            msg = "One or more of the wrap-and-sort options in the salsa-ci file was not supported"
         else:
-            disable_wrap_and_sort = (
-                disable_wrap_and_sort is True or disable_wrap_and_sort == 1
-            )
-        if not disable_wrap_and_sort:
-            wrap_and_sort_options = salsa_ci.mlget(
-                ["variables", "SALSA_CI_WRAP_AND_SORT_ARGS"],
-                list_ok=True,
-                default=None,
-            )
-            if wrap_and_sort_options is None:
-                wrap_and_sort_options = ""
-            elif not isinstance(wrap_and_sort_options, str):
-                return (
-                    None,
-                    None,
-                    "The salsa-ci had a non-string option for wrap-and-sort",
-                )
-            detected_style = parse_salsa_ci_wrap_and_sort_args(wrap_and_sort_options)
-            tool_w_args = f"wrap-and-sort {wrap_and_sort_options}".strip()
-            if detected_style is None:
-                msg = "One or more of the wrap-and-sort options in the salsa-ci file was not supported"
-            else:
-                msg = None
-            return detected_style, tool_w_args, msg
+            msg = None
+        return detected_style, tool_w_args, msg
     if source_package is None:
         return None, None, None
 
