@@ -6,7 +6,6 @@ from typing import Optional, Sequence, List, Tuple
 from debputy import DEBPUTY_ROOT_DIR
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.deb_packaging_support import setup_control_files
-from debputy.dh.debhelper_emulation import dhe_dbgsym_root_dir
 from debputy.filesystem_scan import FSRootDir
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.intermediate_manifest import IntermediateManifest
@@ -21,7 +20,6 @@ from debputy.util import (
     _info,
 )
 
-
 _RRR_DEB_ASSEMBLY_KEYWORD = "debputy/deb-assembly"
 _NOTIFIED_ABOUT_FALLBACK_ASSEMBLY = False
 
@@ -91,12 +89,12 @@ def assemble_debs(
         dbgsym_package_name = f"{package}-dbgsym"
         dctrl_data = package_data_table[package]
         fs_root = dctrl_data.fs_root
-        control_output_dir = assume_not_none(dctrl_data.control_output_dir)
+        control_output_fs_path = dctrl_data.control_output_dir.fs_path
         package_metadata_context = dctrl_data.package_metadata_context
         if (
             dbgsym_package_name in package_data_table
-            or "noautodbgsym" in manifest.build_env.deb_build_options
-            or "noddebs" in manifest.build_env.deb_build_options
+            or "noautodbgsym" in manifest.deb_options_and_profiles.deb_build_options
+            or "noddebs" in manifest.deb_options_and_profiles.deb_build_options
         ):
             # Discard the dbgsym part if it conflicts with a real package, or
             # we were asked not to build it.
@@ -126,7 +124,7 @@ def assemble_debs(
         ):
             # We never built udebs due to #797391. We currently do not generate a control
             # file for it either for the same reason.
-            dbgsym_root = dhe_dbgsym_root_dir(dctrl_bin)
+            dbgsym_root = dctrl_data.dbgsym_info.dbgsym_root_dir
             if not os.path.isdir(output_path):
                 _error(
                     "Cannot produce a dbgsym package when output path is not a directory."
@@ -155,7 +153,7 @@ def assemble_debs(
             deb_materialize,
             intermediate_manifest,
             mtime,
-            control_output_dir,
+            control_output_fs_path,
             output_path,
             upstream_args,
             is_udeb=dctrl_bin.is_udeb,
@@ -171,7 +169,7 @@ def _assemble_deb(
     deb_materialize_cmd: str,
     intermediate_manifest: IntermediateManifest,
     mtime: int,
-    control_output_dir: str,
+    control_output_fs_path: str,
     output_path: str,
     upstream_args: Optional[List[str]],
     is_udeb: bool = False,
@@ -208,7 +206,7 @@ def _assemble_deb(
             "--source-date-epoch",
             str(mtime),
             "--discard-existing-output",
-            control_output_dir,
+            control_output_fs_path,
             materialization_dir,
         ]
     )
@@ -216,7 +214,7 @@ def _assemble_deb(
     if is_udeb:
         materialize_cmd.append("--udeb")
         output = os.path.join(
-            output_path, compute_output_filename(control_output_dir, True)
+            output_path, compute_output_filename(control_output_fs_path, True)
         )
 
     assembly_method = "debputy" if needs_root and use_fallback_assembly else "dpkg-deb"
