@@ -35,8 +35,15 @@ _KNOWN_NON_TYPO_EXTENSIONS = frozenset(
         "bash",
         "pl",
         "py",
+        "md",
         # Fairly common image format in older packages
         "xpm",
+        # Cruft of various kind (for unclean work directories). Just to avoid stupid false-positives.
+        "bak",
+        "tmp",
+        "temp",
+        "orig",
+        "rej",
     }
 )
 
@@ -144,7 +151,8 @@ def _find_definition(
         # We also ignore "foo.1" since manpages are kind of common.
         if not had_arch and (stem in _KNOWN_NON_TYPO_EXTENSIONS or stem.isdigit()):
             continue
-        matches = detect_possible_typo(stem, stems)
+        max_edit_distance = 2 if len(stem) > 3 else 1
+        matches = detect_possible_typo(stem, stems, max_edit_distance=max_edit_distance)
         if matches is not None and len(matches) == 1:
             definition = packager_provided_files[matches[0]]
             return install_as_name, definition, stem
