@@ -161,7 +161,7 @@ class MatchRule:
             _error(
                 f'The pattern "{path_or_glob}" (defined in {definition_source}) looks like it contains a'
                 f' brace expansion (such as "{{a,b}}" or "{{a..b}}").  Brace expansions are not supported.'
-                " If you wanted to match the literal path a brace in it, please use a substitution to insert"
+                " If you wanted to match the literal path with a brace in it, please use a substitution to insert"
                 f' the opening brace.  As an example: "{replacement}"'
             )
 
@@ -526,7 +526,7 @@ class GenericGlobImplementation(MatchRule):
     def _compile_glob(self) -> Sequence[Union[Callable[[str], bool], str]]:
         assert self._glob_pattern.strip("/") == self._glob_pattern
         return [
-            _compile_basename_glob(part) if glob.has_magic(part) else part
+            _compile_basename_glob(part)[1] if glob.has_magic(part) else part
             for part in self._glob_pattern.split("/")
         ]
 
