@@ -624,7 +624,7 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         depends_on_processor: Iterable[str] = tuple(),
         package_type: PackageTypeSelector = "deb",
     ) -> None:
-        self._restricted_api(allowed_plugins={"lua"})
+        self._restricted_api(allowed_plugins={"lua", "debputy-self-hosting"})
         package_processors = self._feature_set.all_package_processors
         dependencies = set()
         processor_key = (self._plugin_name, processor_id)
