@@ -39,12 +39,13 @@ from debputy.packages import BinaryPackage
 from debputy.types import S
 
 if TYPE_CHECKING:
-    from debputy.plugin.debputy.to_be_api_types import BuildSystemRule
-    from debputy.plugin.api.impl_types import DIPHandler
     from debputy.manifest_parser.base_types import (
         StaticFileSystemOwner,
         StaticFileSystemGroup,
     )
+    from debputy.plugin.api.doc_parsing import ParserRefDocumentation
+    from debputy.plugin.api.impl_types import DIPHandler
+    from debputy.plugin.debputy.to_be_api_types import BuildSystemRule
 
 
 DP = TypeVar("DP", bound=DebputyDispatchableType)
@@ -1566,6 +1567,7 @@ def undocumented_attr(attr: str) -> ParserAttributeDocumentation:
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class ParserDocumentation:
+    synopsis: Optional[str] = None
     title: Optional[str] = None
     description: Optional[str] = None
     attribute_doc: Optional[Sequence[ParserAttributeDocumentation]] = None
@@ -1575,6 +1577,25 @@ class ParserDocumentation:
     def replace(self, **changes: Any) -> "ParserDocumentation":
         return dataclasses.replace(self, **changes)
 
+    @classmethod
+    def from_ref_doc(cls, ref_doc: "ParserRefDocumentation") -> "ParserDocumentation":
+        attr = [
+            documented_attr(d["attr"], d["description"])
+            for d in ref_doc.get("attributes", [])
+        ]
+        undoc_attr = ref_doc.get("undocumented_attributes")
+        if undoc_attr:
+            attr.extend(undocumented_attr(attr) for attr in undoc_attr)
+
+        return reference_documentation(
+            title=ref_doc["title"],
+            description=ref_doc["description"],
+            attributes=attr,
+            non_mapping_description=ref_doc.get("non_mapping_description"),
+            reference_documentation_url=ref_doc.get("ref_doc_url"),
+            synopsis=ref_doc.get("synopsis"),
+        )
+
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class TypeMappingExample(Generic[S]):
@@ -1646,6 +1667,7 @@ def reference_documentation(
     attributes: Optional[Sequence[ParserAttributeDocumentation]] = None,
     non_mapping_description: Optional[str] = None,
     reference_documentation_url: Optional[str] = None,
+    synopsis: Optional[str] = None,
 ) -> ParserDocumentation:
     """Provide inline reference documentation for the manifest snippet
 
@@ -1664,6 +1686,7 @@ def reference_documentation(
     :param description: The text you want the user to see as a description for the rule. An auto-generated
       placeholder is provided by default saying that no human written documentation was provided.
       This parameter can be a Python format string with the above listed variables.
+    :param synopsis: One-line plain-text description used for describing the feature during completion.
     :param attributes: A sequence of attribute-related documentation. Each element of the sequence should
       be the result of `documented_attr` or `undocumented_attr`. The sequence must cover all source
       attributes exactly once.
@@ -1675,6 +1698,7 @@ def reference_documentation(
     :return: An opaque representation of the documentation,
     """
     return ParserDocumentation(
+        synopsis,
         title,
         description,
         attributes,
