@@ -1882,7 +1882,7 @@ def test_maintainer_trailing_comma_bug_1089649(line_linter: LintWrapper) -> None
     assert diagnostics and len(diagnostics) == 1
     issue = diagnostics[0]
 
-    msg = 'The "Maintainer" field has a trailing seperator, but it is a single value field.'
+    msg = 'The "Maintainer" field has a trailing separator, but it is a single value field.'
     assert issue.message == msg
     assert f"{issue.range}" == f"4:45-4:46"
     assert issue.severity == DiagnosticSeverity.Error
