@@ -1,16 +1,29 @@
+import dataclasses
 import textwrap
+from typing import Mapping, Dict, Self
 
 import pytest
+from debian.deb822 import Deb822
+from debian.debian_support import DpkgArchTable
 
+from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
+from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
+from debputy.highlevel_manifest import (
+    HighLevelManifest,
+    PathNotCoveredByInstallRulesError,
+)
 from debputy.highlevel_manifest_parser import YAMLManifestParser
 from debputy.installations import (
     InstallSearchDirContext,
     NoMatchForInstallPatternError,
     SearchDir,
 )
-from debputy.plugin.api import virtual_path_def
+from debputy.packages import BinaryPackage, SourcePackage
+from debputy.plugin.api import virtual_path_def, VirtualPath
+from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.spec import INTEGRATION_MODE_DH_DEBPUTY
 from debputy.plugin.api.test_api import build_virtual_file_system
+from debputy.substitution import Substitution
 
 
 @pytest.fixture()
@@ -65,6 +78,72 @@ def manifest_parser_pkg_foo_w_udeb(
     )
 
 
+@dataclasses.dataclass(slots=True, frozen=True)
+class FlexibleManifestParser:
+    dpkg_architecture_variables: DpkgArchitectureBuildProcessValuesTable
+    dpkg_arch_query: DpkgArchTable
+    source_package: SourcePackage
+    substitution: Substitution
+    no_profiles_or_build_options: DebBuildOptionsAndProfiles
+    debputy_plugin_feature_set: PluginProvidedFeatureSet
+    debian_dir: VirtualPath
+    binary_packages: Dict[str, BinaryPackage] = dataclasses.field(default_factory=dict)
+
+    def binary_package(
+        self,
+        fields: Mapping[str, str],
+        *,
+        is_main_package: bool = False,
+        should_be_acted_on: bool = True,
+    ) -> Self:
+        pkg = BinaryPackage(
+            fields,
+            self.dpkg_architecture_variables,
+            self.dpkg_arch_query,
+            is_main_package=is_main_package,
+            should_be_acted_on=should_be_acted_on,
+        )
+        self.binary_packages[pkg.name] = pkg
+        return self
+
+    def parse_manifest(self, manifest_data: str) -> HighLevelManifest:
+        binary_packages = self.binary_packages
+        if not binary_packages:
+            raise ValueError("No binary packages defined!?")
+        return YAMLManifestParser(
+            "debian/test-debputy.manifest",
+            self.source_package,
+            binary_packages,
+            self.substitution,
+            self.dpkg_architecture_variables,
+            self.dpkg_arch_query,
+            self.no_profiles_or_build_options,
+            self.debputy_plugin_feature_set,
+            "full",
+            debian_dir=self.debian_dir,
+        ).parse_manifest(fd=manifest_data)
+
+
+@pytest.fixture()
+def flexible_manifest_parser(
+    amd64_dpkg_architecture_variables: DpkgArchitectureBuildProcessValuesTable,
+    dpkg_arch_query: DpkgArchTable,
+    source_package: SourcePackage,
+    amd64_substitution: Substitution,
+    no_profiles_or_build_options: DebBuildOptionsAndProfiles,
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> FlexibleManifestParser:
+    return FlexibleManifestParser(
+        amd64_dpkg_architecture_variables,
+        dpkg_arch_query,
+        source_package,
+        amd64_substitution,
+        no_profiles_or_build_options,
+        debputy_plugin_feature_set,
+        build_virtual_file_system([]),
+    )
+
+
 def test_install_rules(manifest_parser_pkg_foo) -> None:
     debian_source_root_dir = build_virtual_file_system(
         [virtual_path_def(".", fs_path="/nowhere")]
@@ -1072,3 +1151,199 @@ def test_install_doc_rules_ignore_udeb(manifest_parser_pkg_foo_w_udeb) -> None:
 
     udeb_doc_dir = foo_udeb_fs_root.lookup("/usr/share/doc")
     assert udeb_doc_dir is None
+
+
+def test_bts_1093680_crash_inactive_package(
+    flexible_manifest_parser: FlexibleManifestParser,
+) -> None:
+    debian_source_root_dir = build_virtual_file_system(
+        [
+            virtual_path_def(".", fs_path="/nowhere"),
+            virtual_path_def("examples/", fs_path="/nowhere/examples"),
+            virtual_path_def("examples/case-a/", fs_path="/nowhere/examples/case-a"),
+            virtual_path_def(
+                "examples/case-a/data/", fs_path="/nowhere/examples/case-a/data"
+            ),
+            virtual_path_def(
+                "examples/case-a/data/foo.txt",
+                fs_path="/nowhere/examples/case-a/data/foo.txt",
+                content="data file",
+            ),
+            virtual_path_def("examples/case-b/", fs_path="/nowhere/examples/case-b"),
+            virtual_path_def(
+                "examples/case-b/data/", fs_path="/nowhere/examples/case-b/data"
+            ),
+            virtual_path_def(
+                "examples/case-b/data/foo.txt",
+                fs_path="/nowhere/examples/case-b/data/foo.txt",
+                content="data file",
+            ),
+        ]
+    )
+    manifest_content = textwrap.dedent(
+        """\
+        manifest-version: "0.1"
+        installations:
+          - install-examples:
+              source: "examples/*"
+              into: foo-data
+        """
+    )
+
+    foo_fields = Deb822(
+        {
+            "Package": "foo",
+            "Architecture": "any",
+        }
+    )
+    foo_data_fields = Deb822(
+        {
+            "Package": "foo-data",
+            "Architecture": "all",
+        }
+    )
+    # An arch:any build (-B => foo is active, foo-data is inactive)
+    flexible_manifest_parser.binary_package(foo_fields, is_main_package=True)
+    flexible_manifest_parser.binary_package(foo_data_fields, should_be_acted_on=False)
+
+    manifest = flexible_manifest_parser.parse_manifest(manifest_content)
+    all_pkgs = frozenset(manifest.all_packages)
+
+    manifest.perform_installations(
+        INTEGRATION_MODE_DH_DEBPUTY,
+        install_request_context=InstallSearchDirContext(
+            [
+                SearchDir(debian_source_root_dir, all_pkgs),
+            ],
+            [debian_source_root_dir],
+        ),
+    )
+
+
+@pytest.mark.parametrize(
+    "should_act_on_foo,should_act_on_foo_data",
+    [
+        (True, True),
+        (True, False),
+        (False, False),
+        (False, False),
+    ],
+)
+def test_debputy_132(
+    flexible_manifest_parser: FlexibleManifestParser,
+    should_act_on_foo: bool,
+    should_act_on_foo_data: bool,
+) -> None:
+    debian_source_root_dir = build_virtual_file_system(
+        [
+            virtual_path_def(".", fs_path="/nowhere"),
+            virtual_path_def("built/", fs_path="/nowhere/built"),
+            virtual_path_def("built/data/", fs_path="/nowhere/built/data"),
+            virtual_path_def("built/data/foo.txt", fs_path="/nowhere/data/foo.txt"),
+            virtual_path_def("built/bin/", fs_path="/nowhere/built/bin"),
+            virtual_path_def(
+                "built/bin/foo",
+                fs_path="/nowhere/built/bin/foo",
+                content="#!/bin/sh\n\nexit 0",
+            ),
+        ]
+    )
+    manifest_content = textwrap.dedent(
+        """\
+        manifest-version: "0.1"
+        installations:
+          - install:
+              source: "built/data"
+              dest-dir: "usr/share/foo"
+              into: foo-data
+          - install:
+              source: "built/bin"
+              dest-dir: "usr"
+              into: foo
+        """
+    )
+
+    foo_fields = Deb822(
+        {
+            "Package": "foo",
+            "Architecture": "any",
+        }
+    )
+    foo_data_fields = Deb822(
+        {
+            "Package": "foo-data",
+            "Architecture": "all",
+        }
+    )
+    # An arch:any build (-B => foo is active, foo-data is inactive)
+    flexible_manifest_parser.binary_package(
+        foo_fields,
+        is_main_package=True,
+        should_be_acted_on=should_act_on_foo,
+    )
+    flexible_manifest_parser.binary_package(
+        foo_data_fields,
+        should_be_acted_on=should_act_on_foo_data,
+    )
+
+    manifest = flexible_manifest_parser.parse_manifest(manifest_content)
+    all_pkgs = frozenset(manifest.all_packages)
+
+    manifest.perform_installations(
+        INTEGRATION_MODE_DH_DEBPUTY,
+        install_request_context=InstallSearchDirContext(
+            [
+                SearchDir(debian_source_root_dir, all_pkgs),
+            ],
+            [debian_source_root_dir],
+        ),
+    )
+
+
+def test_uncovered_installation(
+    manifest_parser_pkg_foo: YAMLManifestParser,
+) -> None:
+    debian_source_root_dir = build_virtual_file_system(
+        [
+            virtual_path_def(".", fs_path="/nowhere"),
+            virtual_path_def("usr/", fs_path="/nowhere/debian/tmp/usr"),
+            virtual_path_def("usr/bin/", fs_path="/nowhere/debian/tmp/usr/bin"),
+            virtual_path_def(
+                "usr/bin/foo",
+                fs_path="/nowhere/debian/tmp/usr/bin/foo",
+                content="#!/bin/sh\n\nexit 0",
+            ),
+            virtual_path_def(
+                "usr/bin/bar",
+                fs_path="/nowhere/debian/tmp/usr/bin/bar",
+                content="#!/bin/sh\n\nexit 0",
+            ),
+        ]
+    )
+    manifest_content = textwrap.dedent(
+        """\
+        manifest-version: "0.1"
+        installations:
+          - install:
+              source: "usr/bin/foo"
+              into: foo
+        """
+    )
+
+    manifest = manifest_parser_pkg_foo.parse_manifest(fd=manifest_content)
+    all_pkgs = frozenset(manifest.all_packages)
+
+    with pytest.raises(PathNotCoveredByInstallRulesError) as exec_info:
+        manifest.perform_installations(
+            INTEGRATION_MODE_DH_DEBPUTY,
+            install_request_context=InstallSearchDirContext(
+                [
+                    SearchDir(debian_source_root_dir, all_pkgs),
+                ],
+                [debian_source_root_dir],
+            ),
+        )
+
+    assert len(exec_info.value.unmatched_paths) == 1
+    assert exec_info.value.unmatched_paths[0].path == "./usr/bin/bar"
+    assert exec_info.value.search_dir is debian_source_root_dir
